<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use PLA\Util\Util;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla28000FrequenciaDeProdutosEmGrupoDePratosNoMesSemana extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $crudService;
    protected $parameterBag;
    protected $util;
    protected $dataSourceManager;
    protected $connection;
    protected $RestrContrService;
    protected $errorMessages = array();
    protected $errors = array();

	public function __construct($crudService, $parameterBag, $util, $dataSourceManager, $connection, $RestrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    //Cascade save Vigencia->Servico->Produto
    public function saveVigencia($vigencia, $response, $dataSourceName = "vigfreqprodgrprat") {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveServico($servico, $response) {
        try {
            $dataSourceName = "freqprodgrprat";
            $servico["NRFREQPRODGRPRAT"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrfreqprodgrprat = $rowsToSave[0]["NRFREQPRODGRPRAT"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrfreqprodgrprat;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveProduto(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            if (isset($row["servico"])) {
                $row["servico"]["NRVIGENCIA"] = isset($nrvigencia) ? $nrvigencia : $row["servico"]["NRVIGENCIA"];
                $nrfreqprodgrprat = $this->saveServico($row["servico"], $response);
            }
            $this->dataSourceName = "itfreqprodgrprat";
            foreach ($row["produto"]["NMPRODUTO"] as $produto) {
                $produtoRow = array(
                    "NRITFREQPRODGRPRAT" => $this->RestrContrService->getNewCode($this->dataSourceName),
                    "NRFREQPRODGRPRAT"   => isset($nrfreqprodgrprat) ? $nrfreqprodgrprat : $row["produto"]["NRFREQPRODGRPRAT"],
                    "NRGRUPOPRATOSERV"   => $row["produto"]["NRGRUPOPRATOSERV"],
                    "NRGRUPOPRATO"       => $row["produto"]["NRGRUPOPRATO"],
                    "NRMINPRODGRPRAT"    => $row["produto"]["NRMINPRODGRPRAT"],
                    "NRMAXPRODGRPRAT"    => $row["produto"]["NRMAXPRODGRPRAT"],
                    "NRMAXPRODGRPRATSEM" => $row["produto"]["NRMAXPRODGRPRATSEM"],
                    "CDPRODUTO"          => $produto["CDPRODUTO"],
                    "__is_new"           => true
                );
                
                try {
                    $this->util->validateDuplicatedRecords($produtoRow, "itfreqprodgrprat", array("CDPRODUTO", "NRGRUPOPRATOSERV", "NRFREQPRODGRPRAT"));
                    $dataSet = new DataSet($this->dataSourceName, array($produtoRow));
                    
                    $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                    parent::save($newRequest, $response);
                } catch(\Exception $e) {
                    if ($e->getMessage() == "registerExists") {
                        array_push($this->errorMessages, array("NMPRODUTO" => $produto["NMPRODUTO"]));
                    }
                }
            }

            $this->connection->commit();
            if (!empty($this->errorMessages)) {
                $response->addDataSet(new DataSet("errors", $this->errorMessages));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];
            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }
            $tpoper = $row["TPOPER"];

            if ($tpoper == "S") {
                $this->deleteFreqgrpratmesByNrvigencia($nrvigenciadest);
            }
            foreach ($servicos as $servico) {
                $produtos = $this->getItfreqprodgrprat($servico["NRFREQPRODGRPRAT"]);
                $nrfreqprodgrpratdest = $this->saveServicoImportExport($servico, $row, $nrvigenciadest);
                $produtosExisting = $this->getItfreqprodgrprat($nrfreqprodgrpratdest);
                $produtosExisting = array_map(function($produto){
                    return $produto["NRGRUPOPRATOSERV"].'|'.$produto["CDPRODUTO"];
                }, $produtosExisting);
                $produtosToSave = array();
                foreach ($produtos as &$produto) {
                    if (in_array($produto["NRGRUPOPRATOSERV"].'|'.$produto["CDPRODUTO"], $produtosExisting)) {
                        continue;
                    } else {
                        $produto["NRFREQPRODGRPRAT"] = $nrfreqprodgrpratdest;
                        $produto["NRITFREQPRODGRPRAT"] = $this->RestrContrService->getNewCode("ITFREQPRODGRPRAT");
                        $produto["__is_new"] = true;
                        array_push($produtosToSave, $produto);
                    }
                }
                $dataSourceName = "itfreqprodgrprat";
                $dataSet = new DataSet($dataSourceName, $produtosToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function saveServicoImportExport($servico, $vigencia, $nrvigenciadest) {
        $dataSourceName = "freqprodgrprat";

        $conditions = $this->RestrContrService->buildConditions(array("CDSERVFILI", "NRVIGENCIA"), array($servico["CDSERVFILI"], $nrvigenciadest));
        $freqprodgrprat = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (!empty($freqprodgrprat)) {
            $nrfreqprodgrprat =  $freqprodgrprat[0]["NRFREQPRODGRPRAT"];
        } else {
            $servico["NRVIGENCIA"] = $nrvigenciadest;
            $servico["DTVIGENCIA"] = $vigencia["DTVIGENCIA"];
            $servico["CDFILIAL"] = $vigencia["CDFILIAL"];
            $servico["DSFREQPRODGRPRAT"] = $vigencia["DSFREQPRODGRPRAT"];
            $nrfreqprodgrprat = $this->RestrContrService->getNewCode("FREQPRODGRPRAT");
            $servico["NRFREQPRODGRPRAT"] = $nrfreqprodgrprat;
            $servico["__is_new"] = true;
            
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
        return $nrfreqprodgrprat;
    }

    public function getItfreqprodgrprat($nrfreqprodgrprat) {
        $conditions = $this->RestrContrService->buildConditions(array("NRFREQPRODGRPRAT"), array($nrfreqprodgrprat));
        return $this->crudService->getFilteredDataSet("itfreqprodgrprat", $conditions)->getRows();
    }

    private function deleteFreqgrpratmesByNrvigencia($nrvigencia) {
        $sql = "DELETE FROM FREQPRODGRPRAT WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

        public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    $row = array(
                        "CDFILIAL" => $vigenciaOrig["CDFILIAL"],
                        "DTVIGENCIA" => $vigenciaOrig["DTVIGENCIA"],
                        "DTVIGENCIAFIM" => $vigenciaOrig["DTVIGENCIAFIM"],
                        "DESCRICAO" => $vigenciaOrig["DESCRICAO"],
                        "IDTIPORESTRICAO" => $vigenciaOrig["IDTIPORESTRICAO"],
                        "CDDESCOBSFILIAL" => $vigenciaOrig["CDDESCOBSFILIAL"],
                        "dataSourceName" => strtolower($table),
                        "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA"),
                        "__is_new" => true
                    );
                    try {
                        $this->RestrContrService->validateRegisterVigencia($row);
                        if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                            $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                        }
                        $this->saveVigencia($row, $response, $row["dataSourceName"]);
                    } catch( \Exception $e) {
                        if ($e->getMessage() == "registerExists") {
                            array_push($this->errors, $table);
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            
            $this->connection->commit();
            if (!empty($this->errors)) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("vigenciaExportErrors", 500));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}